@@ -2,15 +2,15 @@
 import itertools
 import logging
 
-from pyhap import util, SUPPORT_QR_CODE
+from pyhap import SUPPORT_QR_CODE, util
 from pyhap.const import (
-    STANDALONE_AID,
+    CATEGORY_BRIDGE,
+    CATEGORY_OTHER,
     HAP_REPR_AID,
     HAP_REPR_IID,
     HAP_REPR_SERVICES,
     HAP_REPR_VALUE,
-    CATEGORY_OTHER,
-    CATEGORY_BRIDGE,
+    STANDALONE_AID,
 )
 from pyhap.iid_manager import IIDManager
 
@@ -25,9 +25,6 @@ class Accessory:
     """A representation of a HAP accessory.
 
     Inherit from this class to build your own accessories.
-
-    At the end of the init of this class, the _set_services method is called.
-    Use this to set your HAP services.
     """
 
     category = CATEGORY_OTHER
@@ -51,7 +48,6 @@ class Accessory:
         self.iid_manager = IIDManager()
 
         self.add_info_service()
-        self._set_services()
 
     def __repr__(self):
         """Return the representation of the accessory."""
@@ -60,19 +56,6 @@ class Accessory:
             self.display_name, services
         )
 
-    def __getstate__(self):
-        state = self.__dict__.copy()
-        state["driver"] = None
-        state["run_sentinel"] = None
-        return state
-
-    def _set_services(self):
-        """Set the services for this accessory.
-
-        .. deprecated:: 2.0
-           Initialize the service inside the accessory `init` method instead.
-        """
-
     @property
     def available(self):
         """Accessory is available.
@@ -107,7 +90,7 @@ class Accessory:
             serv_info.configure_char("Manufacturer", value=manufacturer)
         if model:
             serv_info.configure_char("Model", value=model)
-        if serial_number:
+        if serial_number is not None:
             if len(serial_number) >= 1:
                 serv_info.configure_char("SerialNumber", value=serial_number)
             else:
@@ -133,26 +116,6 @@ class Accessory:
         for service in self.services:
             service.is_primary_service = service.type_id == primary_service.type_id
 
-    def config_changed(self):
-        """Notify the accessory about configuration changes.
-
-        These include new services or updated characteristic values, e.g.
-        the Name of a service changed.
-
-        This method also notifies the driver about the change, so that it can
-        publish the changes to the world.
-
-        .. note:: If you are changing the configuration of a bridged accessory
-           (i.e. an Accessory that is contained in a Bridge),
-           you should call the `config_changed` method on the Bridge.
-
-        Deprecated. Use `driver.config_changed()` instead.
-        """
-        logger.warning(
-            "This method is now deprecated. Use 'driver.config_changed' instead."
-        )
-        self.driver.config_changed()
-
     def add_service(self, *servs):
         """Add the given services to this Accessory.
 
@@ -320,7 +283,7 @@ class Accessory:
 
     # Driver
 
-    def publish(self, value, sender, sender_client_addr=None):
+    def publish(self, value, sender, sender_client_addr=None, immediate=False):
         """Append AID and IID of the sender and forward it to the driver.
 
         Characteristics call this method to send updates.
@@ -336,7 +299,7 @@ class Accessory:
             HAP_REPR_IID: self.iid_manager.get_iid(sender),
             HAP_REPR_VALUE: value,
         }
-        self.driver.publish(acc_data, sender_client_addr)
+        self.driver.publish(acc_data, sender_client_addr, immediate)
 
 
 class Bridge(Accessory):
