@@ -25,6 +25,7 @@ import re
 import socket
 import sys
 import tempfile
+import time
 import threading
 
 from zeroconf import ServiceInfo
@@ -39,6 +40,8 @@ from pyhap.const import (
     HAP_REPR_AID,
     HAP_REPR_CHARS,
     HAP_REPR_IID,
+    HAP_REPR_TTL,
+    HAP_REPR_PID,
     HAP_REPR_STATUS,
     HAP_REPR_VALUE,
     MAX_CONFIG_VERSION,
@@ -252,6 +255,7 @@ class AccessoryDriver:
         listen_address = listen_address or address
         network_tuple = (listen_address, self.state.port)
         self.http_server = HAPServer(network_tuple, self)
+        self.prepared_writes = {}
 
     def start(self):
         """Start the event loop and call `start_service`.
@@ -466,8 +470,9 @@ class AccessoryDriver:
 
         for topic in client_topics:
             self.async_subscribe_client_topic(client, topic, subscribe=False)
+        self.prepared_writes.pop(client, None)
 
-    def publish(self, data, sender_client_addr=None):
+    def publish(self, data, sender_client_addr=None, immediate=False):
         """Publishes an event to the client.
 
         The publishing occurs only if the current client is subscribed to the topic for
@@ -482,14 +487,14 @@ class AccessoryDriver:
             return
 
         if threading.current_thread() == self.tid:
-            self.async_send_event(topic, data, sender_client_addr)
+            self.async_send_event(topic, data, sender_client_addr, immediate)
             return
 
         self.loop.call_soon_threadsafe(
-            self.async_send_event, topic, data, sender_client_addr
+            self.async_send_event, topic, data, sender_client_addr, immediate
         )
 
-    def async_send_event(self, topic, data, sender_client_addr):
+    def async_send_event(self, topic, data, sender_client_addr, immediate):
         """Send an event to a client.
 
         Must be called in the event loop
@@ -513,8 +518,8 @@ class AccessoryDriver:
                     client_addr,
                 )
                 continue
-            logger.debug("Sending event to client: %s", client_addr)
-            pushed = self.http_server.push_event(data, client_addr)
+            logger.debug("Sending event to client: %s, immediate: %s", client_addr, immediate)
+            pushed = self.http_server.push_event(data, client_addr, immediate)
             if not pushed:
                 logger.debug(
                     "Could not send event to %s, probably stale socket.", client_addr
@@ -755,16 +760,29 @@ class AccessoryDriver:
         accessory_callbacks = {}
         setter_results = {}
         had_error = False
+        expired = False
+
+        if HAP_REPR_PID in chars_query:
+            pid = chars_query[HAP_REPR_PID]
+            expire_time = self.prepared_writes.get(client_addr, {}).pop(pid, None)
+            if expire_time is None or time.time() > expire_time:
+                expired = True
 
         for cq in chars_query[HAP_REPR_CHARS]:
             aid, iid = cq[HAP_REPR_AID], cq[HAP_REPR_IID]
-            setter_results.setdefault(aid, {})
+            result = setter_results.setdefault(aid, {})
             char = self.accessory.get_characteristic(aid, iid)
 
+            if expired:
+                result[iid] = HAP_SERVER_STATUS.INVALID_VALUE_IN_REQUEST
+                had_error = True
+                continue
+
             if HAP_PERMISSION_NOTIFY in cq:
                 char_topic = get_topic(aid, iid)
+                action = "Subscribed" if cq[HAP_PERMISSION_NOTIFY] else "Unsubscribed"
                 logger.debug(
-                    "Subscribed client %s to topic %s", client_addr, char_topic
+                    "%s client %s to topic %s", action, client_addr, char_topic
                 )
                 self.async_subscribe_client_topic(
                     client_addr, char_topic, cq[HAP_PERMISSION_NOTIFY]
@@ -784,12 +802,10 @@ class AccessoryDriver:
                     char.display_name,
                     value,
                 )
-                setter_results[aid][
-                    iid
-                ] = HAP_SERVER_STATUS.SERVICE_COMMUNICATION_FAILURE
+                result[iid] = HAP_SERVER_STATUS.SERVICE_COMMUNICATION_FAILURE
                 had_error = True
             else:
-                setter_results[aid][iid] = HAP_SERVER_STATUS.SUCCESS
+                result[iid] = HAP_SERVER_STATUS.SUCCESS
 
             # For some services we want to send all the char value
             # changes at once.  This resolves an issue where we send
@@ -846,6 +862,29 @@ class AccessoryDriver:
             ]
         }
 
+    def prepare(self, prepare_query, client_addr):
+        """Called from ``HAPServerHandler`` when iOS wants to prepare a write.
+
+        :param prepare_query: A prepare query. For example:
+
+        .. code-block:: python
+
+           {
+              "ttl": 10000, # in milliseconds
+              "pid": 12345678,
+           }
+
+        :type prepare_query: dict
+        """
+        try:
+            ttl = prepare_query[HAP_REPR_TTL]
+            pid = prepare_query[HAP_REPR_PID]
+            self.prepared_writes.setdefault(client_addr, {})[pid] = time.time() + (ttl / 1000)
+        except (KeyError, ValueError):
+            return {HAP_REPR_STATUS: HAP_SERVER_STATUS.INVALID_VALUE_IN_REQUEST}
+
+        return {HAP_REPR_STATUS: HAP_SERVER_STATUS.SUCCESS}
+
     def signal_handler(self, _signal, _frame):
         """Stops the AccessoryDriver for a given signal.
 
